[PATCH] D38724: [ELF] - Do not collect SHT_REL[A] sections unconditionally when --gc-sections and --emit-relocs used together.

George Rimar via Phabricator via llvm-commits llvm-commits at lists.llvm.org
Tue Oct 24 01:26:54 PDT 2017


This revision was automatically updated to reflect the committed changes.
Closed by commit rL316418: [ELF] - Do not collect SHT_REL[A] sections unconditionally when --gc-sections… (authored by grimar).

Changed prior to commit:
  https://reviews.llvm.org/D38724?vs=119829&id=120009#toc

Repository:
  rL LLVM

https://reviews.llvm.org/D38724

Files:
  lld/trunk/ELF/InputSection.h
  lld/trunk/ELF/MarkLive.cpp
  lld/trunk/test/ELF/emit-relocs-gc.s


Index: lld/trunk/ELF/MarkLive.cpp
===================================================================
--- lld/trunk/ELF/MarkLive.cpp
+++ lld/trunk/ELF/MarkLive.cpp
@@ -162,20 +162,17 @@
     scanEhFrameSection<ELFT>(EH, EH.template rels<ELFT>(), Fn);
 }
 
-// We do not garbage-collect two types of sections:
-// 1) Sections used by the loader (.init, .fini, .ctors, .dtors or .jcr)
-// 2) Non-allocatable sections which typically contain debugging information
+// Some sections are used directly by the loader, so they should never be
+// garbage-collected. This function returns true if a given section is such
+// section.
 template <class ELFT> static bool isReserved(InputSectionBase *Sec) {
   switch (Sec->Type) {
   case SHT_FINI_ARRAY:
   case SHT_INIT_ARRAY:
   case SHT_NOTE:
   case SHT_PREINIT_ARRAY:
     return true;
   default:
-    if (!(Sec->Flags & SHF_ALLOC))
-      return true;
-
     StringRef S = Sec->Name;
     return S.startswith(".ctors") || S.startswith(".dtors") ||
            S.startswith(".init") || S.startswith(".fini") ||
@@ -198,9 +195,6 @@
     if (Sec == &InputSection::Discarded)
       return;
 
-    // We don't gc non alloc sections.
-    if (!(Sec->Flags & SHF_ALLOC))
-      return;
 
     // Usually, a whole section is marked as live or dead, but in mergeable
     // (splittable) sections, each piece of data has independent liveness bit.
Index: lld/trunk/ELF/InputSection.h
===================================================================
--- lld/trunk/ELF/InputSection.h
+++ lld/trunk/ELF/InputSection.h
@@ -237,8 +237,8 @@
 
   // Mark the piece at a given offset live. Used by GC.
   void markLiveAt(uint64_t Offset) {
-    assert(this->Flags & llvm::ELF::SHF_ALLOC);
-    LiveOffsets.insert(Offset);
+    if (this->Flags & llvm::ELF::SHF_ALLOC)
+      LiveOffsets.insert(Offset);
   }
 
   // Translate an offset in the input section to an offset


-------------- next part --------------
A non-text attachment was scrubbed...
Name: D38724.120009.patch
Type: text/x-patch
Size: 3124 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20171024/0da3eb1b/attachment.bin>


More information about the llvm-commits mailing list
Index: lld/trunk/test/ELF/emit-relocs-gc.s
===================================================================
--- lld/trunk/test/ELF/emit-relocs-gc.s
+++ lld/trunk/test/ELF/emit-relocs-gc.s
@@ -1,18 +1,30 @@
 # REQUIRES: x86
 # RUN: llvm-mc -filetype=obj -triple=x86_64-unknown-linux %s -o %t.o
 
 ## Show that we emit .rela.bar when GC is disabled.
 # RUN: ld.lld --emit-relocs %t.o -o %t
 # RUN: llvm-objdump %t -section-headers | FileCheck %s --check-prefix=NOGC
+# NOGC: .rela.text
 # NOGC: .rela.bar
 
 ## GC collects .bar section and we exclude .rela.bar from output.
 # RUN: ld.lld --gc-sections --emit-relocs --print-gc-sections %t.o -o %t \
 # RUN:   | FileCheck --check-prefix=MSG %s
 # MSG: removing unused section from '.bar' in file
 # MSG: removing unused section from '.rela.bar' in file
 # RUN: llvm-objdump %t -section-headers | FileCheck %s --check-prefix=GC
 # GC-NOT:  rela.bar
+# GC:      rela.text
+# GC-NOT:  rela.bar
 
 .section .bar,"a"
 .quad .bar
+
+.text
+relocs:
+.quad _start
+
+.global _start
+_start:
+ nop
@@ -1,18 +1,30 @@
 # RUN: llvm-mc -filetype=obj -triple=x86_64-unknown-linux %s -o %t.o
 
-## Show that we emit .rela.bar when GC is disabled.
+## Show that we emit .rela.bar and .rela.text when GC is disabled.
 # RUN: ld.lld --emit-relocs %t.o -o %t
 # RUN: llvm-objdump %t -section-headers | FileCheck %s --check-prefix=NOGC
@@ -1,18 +1,30 @@
 # NOGC: .rela.bar
 
-## GC collects .bar section and we exclude .rela.bar from output.
+## GC collects .bar section and we exclude .rela.bar from output. We keep
+## .rela.text because we keep .text.
 # RUN: ld.lld --gc-sections --emit-relocs --print-gc-sections %t.o -o %t \
 # RUN:   | FileCheck --check-prefix=MSG %s